/******************************************************************************
DtK - DJedi Tool Kit. Un tool kit X Window
Copyright (C) 1998-1999 Valvassori Moise (djedi@citeweb.net)

Ce programme est libre, vous pouvez le redistribuer et/ou le modifier selon
les termes de la Licence Publique Gnrale GNU publie par la Free Software
Foundation (version 2 ou bien toute autre version ultrieure choisie par
vous).

Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
GARANTIE, ni explicite ni implicite, y compris les garanties de
commercialisation ou d'adaptation dans un but spcifique. Reportez-vous  la
Licence Publique Gnrale GNU pour plus de dtails.

Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en mme
temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
tats-Unis.
******************************************************************************/
/* DJedi tool box */
/* formular */

#ifndef FORM_H
#define FORM_H
#include "../config.h"
#include "color.h"

typedef enum {normal,in,press,mdisabled}BMode;
typedef enum {left,center,right,align_error}Align;
typedef enum {horizontal,vertical} Direction;

typedef struct{
  int disabled :1;			/* !!! bit field !!! */
}State;

/* Nombre maximum de char dans les inputs */
#define MAX_TEXT 256

typedef struct FORM {
  Window window;
  GC gc;
  enum{twindow,tbutton,tbox,tcheck,tradio,ttext,tibox,tinput,
	 tslider,tslider_button,tsubwindow,tpopup,tpp_item,tpixmap
#ifdef HAVE_LINUX_FB_H
	 ,tframebuffer
#endif
	 } istype;

  struct FORM *parent;
  State state;
  union {
    struct{				/* button */
      char *text;
    } button;

    struct{				/* box */
      BMode mode;
    } box;

    struct{				/* check */
      char *text;
      boolean value;
    } check;

    struct{				/* radio */
      char *text;
      boolean value;
    } radio;

    struct{				/* text */
      char *text;
      Align align;
    } text;

    struct{				/* input */
      char *text;
      int pos;
      int offset;
      boolean focus;
    } input;

    struct{				/* slider */
      Direction direction;
      struct FORM * slider;
    }slider;

    struct{
      int position;
      int size;
    }slider_button;

    struct{				/* Popup */
      struct FORM ** items;
      int n;
    }popup;
    struct{
      char *text;
    }pp_item;

    struct{				/* Pixmap */
      Pixmap pixmap;
    }pixmap;

    struct{				/* Sub Window */     
    } subwindow;

#ifdef HAVE_LINUX_FB_H
    struct{				/* Framebuffer */     
      void * buffer;
      void * fb;
    } framebuffer;
#endif
  }type;				/*** end union ***/

  struct FORM *popup;			/* eventuellement sur chaque objet*/
  void (*action)(struct FORM *i);
  void (*redraw)(struct FORM *i);
  void (*enter)(struct FORM *i);
  void (*leave)(struct FORM *i);
  void (*release)(struct FORM *i);
  void (*press)(struct FORM *i);
  void (*kpress)(struct FORM *i,XKeyEvent * event);
  void (*focus)(struct FORM *i,boolean focus);
  void (*motion)(struct FORM *i);
  int x,y;
  int w,h;
  struct FORM *next;
} Form;

						  /*  Declaration des fonctions externes */
#ifndef DJedi_button
extern Form *button(Form *parent,int x,int y,uint l,uint h,char * title,void (*action)(Form *form));
#endif
#ifndef DJedi_box
extern Form *box(Form *parent,int x,int y,uint l,uint h,BMode mode);
#endif
#ifndef DJedi_ibox
extern Form *ibox(Form *parent,int x,int y,uint l,uint h);
#endif
#ifndef DJedi_check
extern Form *check(Form *parent,int x,int y,uint l,uint h,char *title,boolean value);
extern boolean set_check(Form *f,boolean v);
extern boolean get_check(Form *f);
extern char * set_check_text(Form *f,char *text);
#endif
#ifndef DJedi_radio
extern Form *radio(Form *parent,int x,int y,uint l,uint h,char *title,boolean value);
extern boolean get_radio(Form *f);
extern boolean set_radio(Form *f,boolean v);
extern char * set_radio_text(Form *f,char *text);
#endif
#ifndef DJedi_text
extern Form *text(Form *parent,int x,int y,uint l,uint h,char *text,Align align);
extern char *set_text(Form *f,char *text);
extern Align set_text_justification(Form *f,Align a);
#endif
#ifndef DJedi_input
extern Form *input(Form *parent,int x,int y,uint l,uint h,char *text);
extern char *get_input_text(Form *f);
extern char *set_input_text(Form *f,char *text);
#endif
#ifndef DJedi_form
extern Form *_open_form(Form *parent,int x,int y,uint l,uint h,int bord_widht);
extern void _Box(Form *win,int x,int y,int w,int h,BMode mode);
extern void disable(Form *f);
extern void enable(Form *f);
extern boolean is_able(Form *f);
extern void set_action (Form *f,void (*action)(struct FORM *i));
#endif
#ifndef DJedi_slider
extern Form * slider(Form *win,int x,int y,uint l,uint h,Direction direction,int position, int size,void (*action)(Form *form));
#define hslider(win,x,y,l,h,p,s,a) slider(win,x,y,l,h,horizontal,p,s,a)
#define vslider(win,x,y,l,h,p,s,a) slider(win,x,y,l,h,vertical,p,s,a)
extern int get_slider_position(Form *win);
extern int set_slider_position(Form *win,int position);
extern int get_slider_size(Form *win);
extern int set_slider_size(Form *win,int size);
#endif
#ifndef DJedi_subwindow
extern Form *subwindow(Form *parent,int x,int y,uint l,uint h,void (*redraw)(Form *form));
#endif
#ifndef DJedi_popup
extern Form *popup(Form *parent,int nb_item,uint l);
extern Form *add_item(Form *popup,char *item,void (*action)(struct FORM *i));
extern void set_popup(Form *win,Form * popup);
void active_popup (Form * pp);
#endif
#ifndef DJedi_pixmap
extern Form *pixmap(Form *parent,int x,int y,uint l,uint h);
extern Form *pixmapxpm(Form *parent,int x,int y,const char * file);
extern void flush_pixmap(Form *f,boolean flush_pixmap);
extern XImage * image (Form *parent,int x,int y,uint l,uint h);
#endif
#ifdef HAVE_LINUX_FB_H
#ifndef DJedi_framebuffer
extern Form *framebuffer(Form *parent,int x,int y,uint l,uint h,void (*redraw)(Form *form));
extern void DtKAbsolute(Form * f,int *x,int *y);
#endif
#endif
#endif




