/*   api.c --- Interface DtK pour Guile
 *   
 *   Copyright (C) 2000 Valvassori Mose <djedi@altern.org>
 *   
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *   
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA **/

#include <stdio.h>
#include <guile/gh.h>
#include <dtk/dtk.h>

#define SDTK_API
#include "api.h"
#include "callback.h"

static SCM message_form(Form * win, char * mesg,SCM args);
static int message_window(Form * win, char * mesg,SCM args);
static int message_text(Form * win, char * mesg,SCM args);
static int message_check(Form * win, char * mesg,SCM args);
static int message_radio(Form * win, char * mesg,SCM args);
static SCM message_input(Form * win, char * mesg,SCM args);
static SCM message_slider(Form * win, char * mesg,SCM args);

static BMode SCM2BMode(SCM type);
static Align SCM2Align(SCM type);
static boolean SCM2boolean (SCM bool);


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Creer et ouvre une fentre (elle n'est pas encore visible)

  SCM sdtk_open_window 

  SCM x

  SCM y

  SCM w

  SCM h

  SCM title

  SCM fredraw
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_open_window(SCM x,SCM y,SCM w,SCM h,SCM title,SCM fredraw){
  Form * win=NULL;
  char *tl =gh_scm2newstr(title,NULL);
  char *fr=NULL;
  char tmp[80];


  if (!gh_eq_p(fredraw,SCM_UNDEFINED))
    fr =gh_scm2newstr(fredraw,NULL);

  if (fr==NULL)
    win = open_window(gh_scm2long(x),gh_scm2long(y),
		      gh_scm2ulong(w),gh_scm2ulong(h),
		      tl,default_redraw);
  else
    printf("Pas encore de callback\n");

  if (fr !=NULL)
    free(fr);
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}










/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

  SCM sdtk_button

  SCM parent

  SCM x

  SCM y

  SCM w

  SCM h

  SCM title

  SCM faction
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_button(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM title,SCM faction){
  Form * win;
  char *tl =gh_scm2newstr(title,NULL);
  char tmp[80];

  win = button((Form *)gh_scm2ulong(parent),
	       gh_scm2long(x),gh_scm2long(y),
	       gh_scm2ulong(w),gh_scm2ulong(h),
	       tl,sdtk_callback);

  prepare_callback (win,faction);

  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

  SCM sdtk_box

  SCM parent

  SCM x

  SCM y

  SCM w

  SCM h

  SCM type
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_box(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM type){
  Form * win;
  char tmp[80];
  BMode mode;


  if (gh_eq_p(type,SCM_UNDEFINED))
    mode=normal;
  else
    mode = SCM2BMode(type);

  win = box((Form *)gh_scm2ulong(parent),
	    gh_scm2long(x),gh_scm2long(y),
	    gh_scm2ulong(w),gh_scm2ulong(h),
	    mode);

  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM 
sdtk_ibox

SCM parent

SCM x

SCM y

SCM w

SCM h
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_ibox(SCM parent,SCM x,SCM y,SCM w,SCM h){
  Form * win;
  char tmp[80];

  win = ibox((Form *)gh_scm2ulong(parent),
	     gh_scm2long(x),gh_scm2long(y),
	     gh_scm2ulong(w),gh_scm2ulong(h));

  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

  SCM sdtk_text

  SCM parent

  SCM x

  SCM y

  SCM w

  SCM h

  SCM titre

  SCM align
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_text(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM titre,SCM align){
  Form * win;
  char *t = gh_scm2newstr(titre,NULL);
  char tmp[80];

  win = text((Form *)gh_scm2ulong(parent),
	     gh_scm2long(x),gh_scm2long(y),
	     gh_scm2ulong(w),gh_scm2ulong(h),
	     t,SCM2Align(align));

  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  free(t);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

  SCM sdtk_check

  SCM parent

  SCM x

  SCM y

  SCM w

  SCM h

  SCM titre

  SCM value
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_check(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM titre,SCM value){
  Form * win;
  char *t = gh_scm2newstr(titre,NULL);
  char tmp[80];
  boolean val;

  if (gh_eq_p(value,SCM_UNDEFINED))
    val = false;
  else
    val = SCM2boolean(value);

  win = check((Form *)gh_scm2ulong(parent),
	      gh_scm2long(x),gh_scm2long(y),
	      gh_scm2ulong(w),gh_scm2ulong(h),
	      t,val);

  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  free(t);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM 
sdtk_check

SCM parent

SCM x

SCM y

SCM w

SCM h

SCM titre

SCM value
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_radio(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM titre,SCM value){
  Form * win;
  char *t = gh_scm2newstr(titre,NULL);
  char tmp[80];
  boolean val;

  if (gh_eq_p(value,SCM_UNDEFINED))
    val = false;
  else
    val = SCM2boolean(value);
  
  win = radio ((Form *)gh_scm2ulong(parent),
	       gh_scm2long(x),gh_scm2long(y),
	       gh_scm2ulong(w),gh_scm2ulong(h),
	       t,val);
  
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  free(t);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM 
sdtk_input

SCM parent

SCM x

SCM y

SCM w

SCM h

SCM titre

SCM value
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_input(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM titre){
  Form * win;
  char *t;
  char tmp[80];

  if (gh_eq_p(titre,SCM_UNDEFINED))
    t = NULL;
  else
    t = gh_scm2newstr(titre,NULL);
  
  win = input ((Form *)gh_scm2ulong(parent),
	       gh_scm2long(x),gh_scm2long(y),
	       gh_scm2ulong(w),gh_scm2ulong(h),
	       t);
  
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  free(t);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM 
sdtk_hslider

SCM parent

SCM x

SCM y

SCM w

SCM h

SCM pos

SCM size

SCM action
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_hslider(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM pos,SCM size, SCM action){
  Form * win;
  char tmp[80];

  win = hslider ((Form *)gh_scm2ulong(parent),
		 gh_scm2long(x),gh_scm2long(y),
		 gh_scm2ulong(w),gh_scm2ulong(h),
		 gh_scm2ulong(pos),gh_scm2ulong(size),
		 sdtk_callback);

  if (gh_eq_p(action,SCM_UNDEFINED))
    ;
  else
    prepare_callback (win,action);

  
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM 
sdtk_vslider

SCM parent

SCM x

SCM y

SCM w

SCM h

SCM pos

SCM size

SCM action
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_vslider(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM pos,SCM size, SCM action){
  Form * win;
  char tmp[80];

  win = vslider ((Form *)gh_scm2ulong(parent),
		 gh_scm2long(x),gh_scm2long(y),
		 gh_scm2ulong(w),gh_scm2ulong(h),
		 gh_scm2ulong(pos),gh_scm2ulong(size),
		 sdtk_callback);

  if (gh_eq_p(action,SCM_UNDEFINED))
    ;
  else
    prepare_callback (win,action);

  
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}







static SCM 
sdtk_subwindow(SCM parent,SCM x,SCM y,SCM w,SCM h,SCM redraw){
  Form * win;
  char tmp[80];

  win = subwindow ((Form *)gh_scm2ulong(parent),
		   gh_scm2long(x),gh_scm2long(y),
		   gh_scm2ulong(w),gh_scm2ulong(h),
		   sdtk_callback);

  prepare_callback (win,redraw);
  
  sprintf (tmp,"(lambda ( . messages) (*dtk* %ld messages))",(long)win);
  return (gh_eval_str(tmp));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Fonction qui est renvoyer lors de la creation d'un form.
  C'est une fermeture. On communique avec elle par message.

  SCM manip_object

  SCM id

  SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
manip_object(SCM id,SCM args){
  Form * form = (Form *)gh_scm2long(id);
  char *message;
  int retour_message=0;
  SCM retour;
  args=gh_car(args);
  
  if (gh_null_p(args))
    return (id);  
  else{
    message= gh_symbol2newstr(gh_car(args),NULL);

    retour = message_form(form,message,gh_cdr(args));    
    if (((int)retour != 1) && ((int)retour != 0))
      return (retour);
    
    switch (form->istype){
    case twindow:
      retour_message = message_window(form,message,gh_cdr(args));
      break;
    case ttext:
      retour_message = message_text(form,message,gh_cdr(args));
      break;
    case tcheck:
      retour_message = message_check(form,message,gh_cdr(args));
      break;
    case tradio:
      retour_message = message_radio(form,message,gh_cdr(args));
      break;
    case tinput:
       retour = message_input(form,message,gh_cdr(args));
       if (((int)retour != 1) && ((int)retour != 0))
	 return (retour);
      break;
    case tslider:
       retour = message_slider(form,message,gh_cdr(args));
       if (((int)retour != 1) && ((int)retour != 0))
	 return (retour);
      break;
    default:
      ;
    };
    free(message);
    return (gh_bool2scm(retour_message));
  }
  

  printf("DtK Object type %d\n",form->istype);
  return (SCM_BOOL_F);  
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Rentre dans la boucle d'venement

  SCM sdtk_loop #t
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM 
sdtk_loop(){
  loop();
  return (SCM_BOOL_T);
}


static SCM 
sdtk_em(SCM val){
  double d = gh_scm2double(val);

  return (gh_double2scm(em(d)));
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM
message_form

Form * win

char * mesg

SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM
message_form(Form * win, char * mesg,SCM args){
  boolean r;
  
  if (strcmp(mesg,"disable")==0)
    disable (win);
  else if (strcmp(mesg,"enable")==0)
    enable (win);
  else if (strcmp(mesg,"is-able")==0)
    {
      r=is_able(win);
      if (r==true)
	return SCM_BOOL_T;
      else
	return SCM_BOOL_F;
    }
  else if (strcmp(mesg,"set-action")==0)
    {
      set_action(win,sdtk_callback);
      prepare_callback (win,gh_car(args));
    }
  else
    return (0);
  return (1);
}




/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Gre les mssage envoy  un form de type window

  int message_window

  Form * win

  char * mesg

  SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static int 
message_window(Form * win, char * mesg,SCM args){
  
  if (strcmp(mesg,"view-window")==0)
    view_window(win);
  else
    return (0);
  return (1);
}




/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Gre les mssage envoy  un form de type text

  int message_text

  Form * win

  char * mesg

  SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static int 
message_text(Form * win, char * mesg,SCM args){
  
  if (strcmp(mesg,"set-text")==0)
    set_text(win,gh_scm2newstr(gh_car(args),0));
  else if (strcmp(mesg,"set-text-justication")==0)
    set_text_justification(win,SCM2Align(gh_car(args)));
  else
    return (0);
  return (1);
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

int 
message_check

Form * win

char * mesg

SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static int 
message_check(Form * win, char * mesg,SCM args){
  
  if (strcmp(mesg,"set-check-text")==0)
    set_check_text(win,gh_scm2newstr(gh_car(args),0));
  else if (strcmp(mesg,"set-check")==0)
    set_check(win,SCM2boolean(gh_car(args)));
  else if (strcmp(mesg,"get-check")==0)
    return (get_check(win));
  else
    return (0);
  return (1);
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

int 
message_radio

Form * win

char * mesg

SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static int 
message_radio(Form * win, char * mesg,SCM args){
  
  if (strcmp(mesg,"set-radio-text")==0)
    set_radio_text(win,gh_scm2newstr(gh_car(args),0));
  else if (strcmp(mesg,"set-radio")==0)
    set_radio(win,SCM2boolean(gh_car(args)));
  else if (strcmp(mesg,"get-radio")==0)
    return (get_radio(win));
  else
    return (0);
  return (1);
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

int 
message_input

Form * win

char * mesg

SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM
message_input(Form * win, char * mesg,SCM args){
  char *r;
  
  if (strcmp(mesg,"set-input-text")==0)
    set_input_text(win,gh_scm2newstr(gh_car(args),0));
  else if (strcmp(mesg,"get-input-text")==0)
    {
      r=get_input_text(win);
      return gh_str2scm(r,strlen(r));
    }
  else
    return (0);
  return (1);
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  

SCM
message_slider

Form * win

char * mesg

SCM args
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static SCM
message_slider(Form * win, char * mesg,SCM args){
  int r;
  
  if (strcmp(mesg,"set-slider-position")==0)
    set_slider_position(win,gh_scm2long(gh_car(args)));
  else if (strcmp(mesg,"set-slider-size")==0)
    set_slider_size(win,gh_scm2long(gh_car(args)));
  else if (strcmp(mesg,"get-slider-position")==0)
    {
      r=get_slider_position(win);
      return gh_long2scm(r);
    }
  else if (strcmp(mesg,"get-slider-size")==0)
    {
      r=get_slider_size(win);
      return gh_long2scm(r);
    }
  else
    return (0);
  return (1);
}



/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Convertit un SCM en boolean

  boolean SCM2boolean 

  SCM bool
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static boolean 
SCM2boolean (SCM bool){
  char * c=gh_symbol2newstr(bool,NULL);
  boolean ret=false;

  if (strcmp(c,"true")==0)
    ret=true;
  else if (strcmp(c,"false")==0)
    ret=false;

  free(c);
  return (ret);
}







/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Convertit un SCM en BMode

  BMode SCM2BMode

  SCM type
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static BMode 
SCM2BMode(SCM type){
  char * c=gh_symbol2newstr(type,NULL);
  BMode ret=normal;

  if (strcmp(c,"normal")==0)
    ret=normal;
  else if (strcmp(c,"in")==0)
    ret=in;
  else if (strcmp(c,"press")==0)
    ret=press;
  else if (strcmp(c,"mdisabled")==0)
    ret=mdisabled;

  free(c);
  return (ret); 
}





/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  Convertit un SCM en Align

  Align SCM2Align

  SCM type
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
static Align 
SCM2Align(SCM type){
  char * c=gh_symbol2newstr(type,NULL);
  Align ret=left;

  if (strcmp(c,"left")==0)
    ret=left;
  else if (strcmp(c,"center")==0)
    ret=center;
  else if (strcmp(c,"right")==0)
    ret=right;
  else if (strcmp(c,"align_error")==0)
    ret=align_error;

  free(c);
  return (ret); 
}









static SCM 
sdtk_get_color(SCM color){
  char *col =gh_scm2newstr(color,NULL);

  return (gh_ulong2scm(get_color(col)));
}




static SCM 
sdtk_color(SCM win,SCM col){
  color ((Form *)gh_scm2ulong(win),
	 gh_scm2ulong(col));

  return (SCM_BOOL_T);
}




static SCM 
sdtk_plot(SCM win,SCM x,SCM y){
  plot ((Form *)gh_scm2ulong(win),
	gh_scm2ulong(x),gh_scm2ulong(y));

  return (SCM_BOOL_T);
}

static SCM 
sdtk_line(SCM win,SCM x,SCM y,SCM x1,SCM y1){
  line ((Form *)gh_scm2ulong(win),
	gh_scm2ulong(x),gh_scm2ulong(y),
	gh_scm2ulong(x1),gh_scm2ulong(y1));
  
  return (SCM_BOOL_T);
}

static SCM 
sdtk_rect(SCM win,SCM x,SCM y,SCM x1,SCM y1){
  rect ((Form *)gh_scm2ulong(win),
	gh_scm2ulong(x),gh_scm2ulong(y),
	gh_scm2ulong(x1),gh_scm2ulong(y1));
  
  return (SCM_BOOL_T);
}

static SCM 
sdtk_rectf(SCM win,SCM x,SCM y,SCM x1,SCM y1){
  rectf ((Form *)gh_scm2ulong(win),
	 gh_scm2ulong(x),gh_scm2ulong(y),
	 gh_scm2ulong(x1),gh_scm2ulong(y1));
  
  return (SCM_BOOL_T);
}

static SCM 
sdtk_circle(SCM win,SCM x,SCM y,SCM r){
  circle ((Form *)gh_scm2ulong(win),
	  gh_scm2ulong(x),gh_scm2ulong(y),
	  gh_scm2ulong(r));
  
  return (SCM_BOOL_T);
}


static SCM 
sdtk_circlef(SCM win,SCM x,SCM y,SCM r){
  circlef ((Form *)gh_scm2ulong(win),
	  gh_scm2ulong(x),gh_scm2ulong(y),
	  gh_scm2ulong(r));
  
  return (SCM_BOOL_T);
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void
record_functions()
{
  gh_new_procedure("dtk-open-window",sdtk_open_window,5,1,0);
  gh_new_procedure("dtk-subwindow",sdtk_subwindow,6,0,0);
  gh_new_procedure("dtk-button",sdtk_button,7,0,0);
  gh_new_procedure("dtk-box",sdtk_box,5,1,0);
  gh_new_procedure("dtk-ibox",sdtk_ibox,5,0,0);
  gh_new_procedure("dtk-text",sdtk_text,7,0,0);
  gh_new_procedure("dtk-check",sdtk_check,6,1,0);
  gh_new_procedure("dtk-radio",sdtk_radio,6,1,0);
  gh_new_procedure("dtk-input",sdtk_input,5,1,0);
  gh_new_procedure("dtk-hslider",sdtk_hslider,7,1,0);
  gh_new_procedure("dtk-vslider",sdtk_vslider,7,1,0);
  gh_new_procedure("*dtk*",manip_object,1,0,1);
  gh_new_procedure("dtk-loop",sdtk_loop,0,0,0);
  gh_new_procedure("em",sdtk_em,1,0,0);

  gh_new_procedure("dtk-get-color",sdtk_get_color,1,0,0);
  gh_new_procedure("dtk-color",sdtk_color,2,0,0);
  gh_new_procedure("dtk-plot",sdtk_plot,3,0,0);
  gh_new_procedure("dtk-line",sdtk_line,5,0,0);
  gh_new_procedure("dtk-rect",sdtk_rect,5,0,0);
  gh_new_procedure("dtk-rectf",sdtk_rectf,5,0,0);
  gh_new_procedure("dtk-circle",sdtk_circle,4,0,0);
  gh_new_procedure("dtk-circlef",sdtk_circlef,4,0,0);
}
